@@ -8,6 +8,7 @@ from gym import spaces
 import numpy as np
 from pettingzoo.utils._parallel_env import _parallel_env_wrapper
 from pettingzoo.utils.to_parallel import parallel_wrapper_fn
+from pettingzoo.utils.env import ParallelEnv
 
 
 def base_env_wrapper_fn(raw_env_fn):
@@ -24,7 +25,7 @@ def BaseAtariEnv(**kwargs):
     return _parallel_env_wrapper(ParallelAtariEnv(**kwargs))
 
 
-class ParallelAtariEnv(EzPickle):
+class ParallelAtariEnv(ParallelEnv, EzPickle):
 
     metadata = {'render.modes': ['human']}
 
@@ -59,10 +60,6 @@ class ParallelAtariEnv(EzPickle):
         multi_agent_ale_py.ALEInterface.setLoggerMode("error")
         self.ale = multi_agent_ale_py.ALEInterface()
 
-        if seed is None:
-            seed = seeding.create_seed(seed, max_bytes=4)
-
-        self.ale.setInt(b"random_seed", seed)
         self.ale.setFloat(b'repeat_action_probability', 0.)
 
         pathstart = os.path.dirname(multi_agent_ale_py.__file__)
@@ -70,7 +67,8 @@ class ParallelAtariEnv(EzPickle):
         if not os.path.exists(final_path):
             raise IOError("rom {} is not installed. Please install roms using AutoROM tool (https://github.com/PettingZoo-Team/AutoROM)".format(game))
 
-        self.ale.loadROM(final_path)
+        self.rom_path = final_path
+        self.ale.loadROM(self.rom_path)
 
         all_modes = self.ale.getAvailableModes(num_players)
 
@@ -80,7 +78,8 @@ class ParallelAtariEnv(EzPickle):
             mode = mode_num
             assert mode in all_modes, "mode_num parameter is wrong. Mode {} selected, only {} modes are supported".format(mode_num, str(list(all_modes)))
 
-        self.ale.setMode(mode)
+        self.mode = mode
+        self.ale.setMode(self.mode)
         assert num_players == self.ale.numPlayersActive()
 
         if full_action_space:
@@ -106,16 +105,25 @@ class ParallelAtariEnv(EzPickle):
         player_names = ["first", "second", "third", "fourth"]
         self.agents = [f"{player_names[n]}_0" for n in range(self.num_agents)]
 
-        self.action_spaces = [gym.spaces.Discrete(action_size)] * self.num_agents
-        self.observation_spaces = [observation_space] * self.num_agents
+        self.action_spaces = {agent: gym.spaces.Discrete(action_size) for agent in self.agents}
+        self.observation_spaces = {agent: observation_space for agent in self.agents}
 
         self._screen = None
+        self.seed(seed)
+
+    def seed(self, seed=None):
+        if seed is None:
+            seed = seeding.create_seed(seed, max_bytes=4)
+        self.ale.setInt(b"random_seed", seed)
+        self.ale.loadROM(self.rom_path)
+        self.ale.setMode(self.mode)
 
     def reset(self):
         self.ale.reset_game()
         self.frame = 0
 
-        return [self._observe()] * self.num_agents
+        obs = self._observe()
+        return {agent: obs for agent in self.agents}
 
     def _observe(self):
         if self.obs_type == 'ram':
@@ -126,20 +134,25 @@ class ParallelAtariEnv(EzPickle):
         elif self.obs_type == 'grayscale_image':
             return self.ale.getScreenGrayscale()
 
-    def step(self, actions):
-        actions = np.asarray(actions)
+    def step(self, action_dict):
+        actions = np.zeros(self.num_agents, dtype=np.int32)
+        for i, agent in enumerate(self.agents):
+            actions[i] = action_dict[agent]
+
         actions = self.action_mapping[actions]
         rewards = self.ale.act(actions)
         if self.ale.game_over() or self.frame >= self.max_frames:
-            dones = [True] * self.num_agents
+            dones = {agent: True for agent in self.agents}
         else:
             lives = self.ale.allLives()
             # an inactive agent in ale gets a -1 life.
-            dones = [int(life) < 0 for life in lives]
+            dones = {agent: int(life) < 0 for agent, life in zip(self.agents, lives)}
 
         self.frame += 1
-        observations = [self._observe()] * self.num_agents
-        infos = [{}] * self.num_agents
+        obs = self._observe()
+        observations = {agent: obs for agent in self.agents}
+        rewards = {agent: rew for agent, rew in zip(self.agents, rewards)}
+        infos = {agent: {} for agent in self.agents}
         return observations, rewards, dones, infos
 
     def render(self):
