@@ -446,7 +446,7 @@ class MultiWalkerEnv():
         self.frames = self.frames + 1
         if is_last:
             rewards, done, mod_obs = self.scroll_subroutine()
-            self.last_obs[agent_id] = mod_obs[agent_id]
+            self.last_obs = mod_obs
             global_reward = rewards.mean()
             local_reward = rewards * self.local_ratio
             self.last_rewards = global_reward * (1. - self.local_ratio) + local_reward * self.local_ratio
